import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/device_managment/all_devices/bloc/device_mgmt_bloc/device_managment_bloc.dart';
import 'package:syncrow_web/pages/device_managment/all_devices/bloc/switch_tabs/switch_tabs_bloc.dart';
import 'package:syncrow_web/pages/device_managment/all_devices/widgets/device_managment_body.dart';
import 'package:syncrow_web/pages/device_managment/shared/navigate_home_grid_view.dart';
import 'package:syncrow_web/pages/routiens/view/create_new_routine_view.dart';
import 'package:syncrow_web/pages/routiens/view/routines_view.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';
import 'package:syncrow_web/utils/helpers/responsice_layout_helper/responsive_layout_helper.dart';
import 'package:syncrow_web/web_layout/web_scaffold.dart';

class DeviceManagementPage extends StatelessWidget with HelperResponsiveLayout {
  const DeviceManagementPage({super.key});

  @override
  Widget build(BuildContext context) {
    return MultiBlocProvider(
      providers: [
        BlocProvider(
          create: (context) =>
              SwitchTabsBloc()..add(const TriggerSwitchTabsEvent(false)),
        ),
        BlocProvider(
          create: (context) => DeviceManagementBloc()..add(FetchDevices()),
        ),
      ],
      child: WebScaffold(
        appBarTitle: FittedBox(
          child: Text(
            'Device Management',
            style: Theme.of(context).textTheme.headlineLarge,
          ),
        ),
        centerBody: BlocBuilder<SwitchTabsBloc, SwitchTabsState>(
            builder: (context, state) {
          return Row(
            mainAxisSize: MainAxisSize.min,
            children: [
              TextButton(
                style: TextButton.styleFrom(
                  backgroundColor: null,
                ),
                onPressed: () {
                  context
                      .read<SwitchTabsBloc>()
                      .add(const TriggerSwitchTabsEvent(false));
                },
                child: Text(
                  'Devices',
                  style: context.textTheme.titleMedium?.copyWith(
                    color:
                        state is SelectedTabState && state.selectedTab == false
                            ? ColorsManager.whiteColors
                            : ColorsManager.grayColor,
                    fontWeight: (state is SelectedTabState) &&
                            state.selectedTab == false
                        ? FontWeight.w700
                        : FontWeight.w400,
                  ),
                ),
              ),
              TextButton(
                style: TextButton.styleFrom(
                  backgroundColor: null,
                ),
                onPressed: () {
                  context
                      .read<SwitchTabsBloc>()
                      .add(const TriggerSwitchTabsEvent(true));
                },
                child: Text(
                  'Routines',
                  style: context.textTheme.titleMedium?.copyWith(
                    color:
                        (state is SelectedTabState) && state.selectedTab == true
                            ? ColorsManager.whiteColors
                            : ColorsManager.grayColor,
                    fontWeight:
                        (state is SelectedTabState) && state.selectedTab == true
                            ? FontWeight.w700
                            : FontWeight.w400,
                  ),
                ),
              ),
            ],
          );
        }),
        rightBody: const NavigateHomeGridView(),
        scaffoldBody: BlocBuilder<SwitchTabsBloc, SwitchTabsState>(
            builder: (context, state) {
          if (state is SelectedTabState && state.selectedTab) {
            return const RoutinesView();
          }
          if (state is ShowCreateRoutineState && state.showCreateRoutine) {
            return  CreateNewRoutineView();
          }

          return BlocBuilder<DeviceManagementBloc, DeviceManagementState>(
            builder: (context, deviceState) {
              if (deviceState is DeviceManagementLoading) {
                return const Center(child: CircularProgressIndicator());
              } else if (deviceState is DeviceManagementLoaded) {
                return DeviceManagementBody(devices: deviceState.devices);
              } else if (deviceState is DeviceManagementFiltered) {
                return DeviceManagementBody(
                    devices: deviceState.filteredDevices);
              } else {
                return const Center(child: Text('Error fetching Devices'));
              }
            },
          );
        }),
      ),
    );
  }
}
