import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/device_managment/all_devices/bloc/switch_tabs/switch_tabs_bloc.dart';
import 'package:syncrow_web/pages/routiens/bloc/routine_bloc/routine_bloc.dart';
import 'package:syncrow_web/pages/routiens/view/create_new_routine_view.dart';
import 'package:syncrow_web/pages/routiens/widgets/main_routine_view/fetch_routine_scenes_automation.dart';
import 'package:syncrow_web/pages/routiens/widgets/main_routine_view/routine_view_card.dart';
import 'package:syncrow_web/utils/color_manager.dart';

class RoutinesView extends StatefulWidget {
  const RoutinesView({super.key});

  @override
  State<RoutinesView> createState() => _RoutinesViewState();
}

class _RoutinesViewState extends State<RoutinesView> {
  @override
  void initState() {
    super.initState();
    context.read<RoutineBloc>().add(FetchDevicesInRoutine());
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<SwitchTabsBloc, SwitchTabsState>(
      builder: (context, state) {
        if (state is ShowCreateRoutineState && state.showCreateRoutine) {
          return const CreateNewRoutineView(
           
          );
        }
        return Padding(
          padding: const EdgeInsets.all(16),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            crossAxisAlignment: CrossAxisAlignment.start,
            mainAxisAlignment: MainAxisAlignment.start,
            children: [
              Text(
                "Create New Routines",
                style: Theme.of(context).textTheme.titleLarge?.copyWith(
                      color: ColorsManager.grayColor,
                      fontWeight: FontWeight.bold,
                    ),
              ),
              const SizedBox(
                height: 10,
              ),
              RoutineViewCard(
                onTap: () {
                  BlocProvider.of<SwitchTabsBloc>(context).add(
                    const CreateNewRoutineViewEvent(true),
                  );
                  context.read<RoutineBloc>().add(
                        (ResetRoutineState()),
                      );
                },
                icon: Icons.add,
                textString: '',
              ),
              const SizedBox(
                height: 15,
              ),
              const Expanded(child: FetchRoutineScenesAutomation()),
            ],
          ),
        );
      },
    );
  }
}
